package org.greenstone.gsdl3.core;

import java.util.*;
import java.io.*;

import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 
import org.w3c.dom.Node; 

import org.greenstone.gsdl3.util.*;

import javax.xml.xpath.*;

public class SkinnedReceptionist extends NZDLReceptionist {
	
    /** a transformer class to transform xml using xslt */
    protected XMLTransformer transformer=null;	
	
    public SkinnedReceptionist() {
	super();
	System.out.println("Skinned Receptionist created");
	//this.xslt_map = new HashMap();
	this.transformer = new XMLTransformer();
    }
	
    protected Node postProcessPage(Element p) {
	try {
	    //Avoid the duplicate attribute in the namespaces declaration of the generated html file
	    //String NAMESPACE_URI = "http://www.greenstone.org/greenstone3/schema/gsf";
	    //String NAMESPACE_PREFIX = "gsf";
	    //p.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:" + NAMESPACE_PREFIX, NAMESPACE_URI);
			
			
	    Page page = new Page(p, this);
	    Skin skin = new Skin(page, this);
			
	    addExtraInfo(p);
	    addCssFiles(page, skin);
	    addJsFiles(page, skin);
	    addUiElements(page);
	    addMetadataElements(page);
			
	    String output = page.getPageRequest().getAttribute(GSXML.OUTPUT_ATT);
			
	    // might need to add some data to the page
	    //addExtraInfo(page);
			
			
	    System.out.println("action = " + page.getAction());
	    //	if we are displaying a document, specify an attribute for weither a cover image exists.
	    if (page.getAction().equals("d")) {
				
		System.out.println("doing the document thing");
				
		Element document = (Element)p.getElementsByTagName("document").item(0);
		Element metadataList = (Element)GSXML.getChildByTagName(document, "metadataList");
		String archiveDir = "";
		NodeList metadata = metadataList.getChildNodes();
		for (int i=0; i<metadata.getLength(); i++) {
		    Element m = (Element) metadata.item(i);
		    if (m.getAttribute("name").equals("archivedir"))
			archiveDir = GSXML.getNodeText(m);
		}
				
		String coverImage_path = page.getCollectionHome() 
		    + File.separatorChar + "index"  
		    + File.separatorChar + "assoc"  
		    + File.separatorChar + archiveDir  
		    + File.separatorChar + "cover.jpg";		
				
		document.setAttribute("coverImage", coverImage_path);
				
		File coverImage = new File(coverImage_path);
		document.setAttribute("hasCoverImage", String.valueOf(coverImage.canRead()));
	    }
			
			
	    Node np = null;
	    try {

		if (skin != null && !output.equals("xml")) {
		    System.err.println("Away to to skin transformPage");
		    np = skin.transformPage(page);
		    System.err.println("done");

					
		} 
		else {
		    np = p;
		}
	    } catch (Exception e) {
		e.printStackTrace();
	    }
			
			
			
	    return np;
	} catch (Exception e) {
	    e.printStackTrace();
	    return doc.createElement("page");
	}
    }
	
    /** overwrite this to add any extra info that might be needed in the page before transformation */
    protected void addExtraInfo(Element page) {

	//logger.debug("the  page before adding extra info is");
	//logger.debug(this.converter.getPrettyString(page));
		
	super.addExtraInfo(page);
		
	// this gets the collection info
	Element page_response = (Element)GSXML.getChildByTagName(page, GSXML.PAGE_RESPONSE_ELEM);
	Element page_request = (Element)GSXML.getChildByTagName(page, GSXML.PAGE_REQUEST_ELEM);
	Element collection_info = (Element)GSXML.getChildByTagName(page_response, GSXML.COLLECTION_ELEM);
	if (collection_info== null) {
	    return;
	}
		
	String collection = collection_info.getAttribute("name"); // should we take this from the request instead??
	// look through services and see if classifier one is present - need to
	// get the list of classifiers
	Element service_list = (Element)GSXML.getChildByTagName(collection_info, GSXML.SERVICE_ELEM+GSXML.LIST_MODIFIER);
	if (service_list == null) {
	    // something weird has gone wrong
	    return;
	}
		
	Element info_message = this.doc.createElement(GSXML.MESSAGE_ELEM);
	String lang = page.getAttribute(GSXML.LANG_ATT);
		
	boolean do_classifier = false;
	boolean do_query = false;
	Element classifier_service = GSXML.getNamedElement(service_list, GSXML.SERVICE_ELEM, GSXML.NAME_ATT, "ClassifierBrowse");
	Element query_service = GSXML.getNamedElement(service_list, GSXML.SERVICE_ELEM, GSXML.NAME_ATT, "TextQuery");
	if (classifier_service != null) {
	    do_classifier = true;
	    // find the list of classifiers
			
	    String to = GSPath.appendLink(collection, "ClassifierBrowse");
	    Element info_request = GSXML.createBasicRequest(this.doc, GSXML.REQUEST_TYPE_DESCRIBE, to, lang, ""); 
	    info_message.appendChild(info_request);
	}
		
	String action = page_request.getAttribute(GSXML.ACTION_ATT);
	String subaction = page_request.getAttribute(GSXML.SUBACTION_ATT);
		
	if (query_service != null && action.equals("p") && subaction.equals("about")) {
	    do_query = true;
	    // get the full description
	    String to = GSPath.appendLink(collection, "TextQuery");
	    Element query_request = GSXML.createBasicRequest(this.doc, GSXML.REQUEST_TYPE_DESCRIBE, to, lang, "");
	    info_message.appendChild(query_request);
			
	}
		
	Element info_resp_message = (Element) this.mr.process(info_message);
	Element info_response = (Element) GSXML.getChildByTagName(info_resp_message, GSXML.RESPONSE_ELEM); 
	//	String path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.SERVICE_ELEM);
	if (do_classifier) {
	    Element classifier_service_info = (Element)GSXML.getNamedElement(info_response, GSXML.SERVICE_ELEM, GSXML.NAME_ATT, "CLassifierBrowse");
	    if (classifier_service_info != null) {
		service_list.replaceChild(this.doc.importNode(classifier_service_info, true), classifier_service);
	    }
	}
	if (do_query) {
	    Element query_service_info = (Element)GSXML.getNamedElement(info_response, GSXML.SERVICE_ELEM, GSXML.NAME_ATT, "TextQuery");
	    if (query_service_info != null) {
		service_list.replaceChild(this.doc.importNode(query_service_info, true), query_service);
	    }
	}
	//logger.debug("the final page before transforming is");
	//logger.debug(this.converter.getPrettyString(page));
	return;
    }
	
    protected void addCssFiles(Page page, Skin skin) {
		
	Element cssFileList = doc.createElement("cssFileList");
		
	// get skin level files
	if (skin != null) {
	    File dir = new File(skin.rootDirectory.getPath() + File.separatorChar + "css");
	    if (dir.isDirectory()) {
		File[] files = dir.listFiles();
				
		for (int fi=0, fn=files.length; fi<fn; fi++) {
		    if (files[fi].getName().endsWith(".css")) {
			Element cssFile = doc.createElement("cssFile");
			String path = files[fi].getPath();
			path = "." + path.substring(GlobalProperties.getGSDL3Home().length());
			//changes to a correct path in case GS3 runs on Windows
			path = path.replace( '\\', '/' );	
			cssFile.setAttribute("path", path);
			cssFileList.appendChild(cssFile);
		    }
		}
	    }			
	}	
		
	// get site level files
	String site_home = page.getSiteHome();
	File dir = new File(site_home + File.separatorChar + "ui" + File.separatorChar + "css");
	if (dir.isDirectory()) {
	    File[] files = dir.listFiles();
	    for (int fi=0, fn=files.length; fi<fn; fi++) {
		if (files[fi].getName().endsWith(".css")) {
		    Element cssFile = doc.createElement("cssFile");
		    String path = files[fi].getPath();
		    path = "." + path.substring(GlobalProperties.getGSDL3Home().length());
		    //changes to a correct path in case GS3 runs on Windows
		    path = path.replace( '\\', '/' );					
		    cssFile.setAttribute("path", path);
		    cssFileList.appendChild(cssFile);
		}
	    }
	}
		
	// get collect level files
	String collect_home = page.getCollectionHome();
	if (collect_home != "") {
	    dir = new File(collect_home + File.separatorChar + "ui" + File.separatorChar + "css");
	    if (dir.isDirectory()) {
		File[] files = dir.listFiles();
		for (int fi=0, fn=files.length; fi<fn; fi++) {
		    if (files[fi].getName().endsWith(".css")) {
			Element cssFile = doc.createElement("cssFile");
			String path = files[fi].getPath();
			path = "." + path.substring(GlobalProperties.getGSDL3Home().length());
			//changes to a correct path in case GS3 runs on Windows
			path = path.replace( '\\', '/' );		
			cssFile.setAttribute("path", path);
			cssFileList.appendChild(cssFile);
		    }
		}
	    }
	}
		
	page.getPageResponse().appendChild(cssFileList);
    }
	
    protected void addJsFiles(Page page, Skin skin) {
		
	Element jsFileList = doc.createElement("jsFileList");
		
	// get skin level files
	if (skin != null) {
	    File dir = new File(skin.rootDirectory.getPath() + File.separatorChar + "js");
	    if (dir.isDirectory()) {
		File[] files = dir.listFiles();
				
		for (int fi=0, fn=files.length; fi<fn; fi++) {
		    if (files[fi].getName().endsWith(".js")) {
			Element jsFile = doc.createElement("jsFile");
			String path = files[fi].getPath();
			path = "." + path.substring(GlobalProperties.getGSDL3Home().length());
			//changes to a correct path in case GS3 runs on Windows
			path = path.replace( '\\', '/' );	
			jsFile.setAttribute("path", path);
			jsFileList.appendChild(jsFile);
		    }
		}
	    }			
	}	
		
	// get site level files
	String site_home = page.getSiteHome();
	File dir = new File(site_home + File.separatorChar + "ui" + File.separatorChar + "js");
	if (dir.isDirectory()) {
	    File[] files = dir.listFiles();
	    for (int fi=0, fn=files.length; fi<fn; fi++) {
		if (files[fi].getName().endsWith(".js")) {
		    Element jsFile = doc.createElement("jsFile");
		    String path = files[fi].getPath();
		    path = "." + path.substring(GlobalProperties.getGSDL3Home().length());
		    //changes to a correct path in case GS3 runs on Windows
		    path = path.replace( '\\', '/' );	
		    jsFile.setAttribute("path", path);
		    jsFileList.appendChild(jsFile);
		}
	    }
	}
		
	// get collect level files
	String collect_home = page.getCollectionHome();
	if (collect_home != "") {
	    dir = new File(collect_home + File.separatorChar + "ui" + File.separatorChar + "js");
	    if (dir.isDirectory()) {
		File[] files = dir.listFiles();
		for (int fi=0, fn=files.length; fi<fn; fi++) {
		    if (files[fi].getName().endsWith(".js")) {
			Element jsFile = doc.createElement("jsFile");
			String path = files[fi].getPath();
			path = "." + path.substring(GlobalProperties.getGSDL3Home().length());
			//changes to a correct path in case GS3 runs on Windows
			path = path.replace( '\\', '/' );	
			jsFile.setAttribute("path", path);
			jsFileList.appendChild(jsFile);
		    }
		}
	    }
	}
		
	page.getPageResponse().appendChild(jsFileList);
    }
	
    protected void addUiElements(Page page) {
		
	Document siteUi = page.getSiteUi();
	Document collectUi = page.getCollectUi();
	String lang = page.getLanguage();
		
	HashMap elements = getRelevantElements("uiItem", lang, siteUi, collectUi);
	Element xmlElements = doc.createElement("uiItemList");
		
	Iterator i = elements.values().iterator();
	while (i.hasNext()) {
	    Element element = (Element)i.next();
			
	    // doc.adoptNode(element); // Java 1.5 (DOM Level 3)
	    // xmlElements.appendChild(element);

	    // Java 1.4 equivalent
	    Node doc_owned_node = doc.importNode(element,true);
	    // Should also remove element from its original parent, as adopt does?
	    // element.getParentNode().removeChild(element);


	    xmlElements.appendChild(doc_owned_node);
			
			
	}
		
	page.getPageResponse().appendChild(xmlElements);
    }
	
    protected void addMetadataElements(Page page) {
	Document siteMetadata = page.getSiteMetadata();
	Document collectMetadata = page.getCollectMetadata();
	String lang = page.getLanguage();
		
	HashMap elements = getRelevantElements("metadataItem", lang, siteMetadata, collectMetadata);
	Element xmlElements = doc.createElement("metadataList");
		
	Iterator i = elements.values().iterator();
	while (i.hasNext()) {
	    Element element = (Element)i.next();
	    
	    // doc.adoptNode(element); // Java 1.5 (DOM Level 3)
	    // xmlElements.appendChild(element);

	    // Java 1.4 equivalent
	    Node doc_owned_node = doc.importNode(element,true);
	    // Should also remove element from its original parent, as adopt does?
	    // element.getParentNode().removeChild(element);

	    xmlElements.appendChild(doc_owned_node);
	}
		
	page.getPageResponse().appendChild(xmlElements);
    }
	
    protected Node transformPage(Element page) {
	return super.transformPage(page);
    }
	
    private HashMap getRelevantElements(String tagName, String lang, Document siteLevel, Document collectLevel) {
		
	HashMap elements = new HashMap();
		
	// get all relevant elements from collect level
	if (collectLevel != null) {
			
	    NodeList xmlElements = collectLevel.getElementsByTagName(tagName);
	    for (int di = 0, dn = xmlElements.getLength(); di < dn; di++) {
				
		Element xmlElement = (Element)xmlElements.item(di);
				
		// only include elements that are language independent or are the correct language, and do not specify inherit=true
		if ((xmlElement.getAttribute("lang").equals("") || xmlElement.getAttribute("lang").equalsIgnoreCase(lang)) && !xmlElement.getAttribute("inherit").equalsIgnoreCase("true")) {
		    String name = xmlElement.getAttribute("name");
		    //System.out.println(name + " found and added at collect level");
		    elements.put(name, xmlElement);
		}	
	    }
	}
		
	// get all elements from site level
	if (siteLevel != null) {
			
	    //System.out.println("getting all " + tagName + "'s (" + lang + ") from " + converter.getPrettyString(siteLevel));
			
	    NodeList xmlElements = siteLevel.getElementsByTagName(tagName);
	    for (int di = 0, dn = xmlElements.getLength(); di < dn; di++) {
				
		Element xmlElement = (Element)xmlElements.item(di);
				
		// only include elements that are language independent or are the correct language, and which havent already been specified at collect level
		if (xmlElement.getAttribute("lang").equals("") || xmlElement.getAttribute("lang").equalsIgnoreCase(lang)) {
		    String name = xmlElement.getAttribute("name");
		    //System.out.println(name + " found at site level");
					
		    if (!elements.containsKey(name)) {
			elements.put(name, xmlElement);
			//System.out.println(name + " added at site level");
		    }
		}
	    }
	}
		
	return elements;
    }
	
    	
}
